import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.AffineTransform;
import java.awt.Rectangle;

/**
 * Eine Moebelgruppe , die manipuliert werden kann und sich selbst auf einer Leinwand zeichnet.
 * @author  claus albowski
 * @version 2.0 (Nov 05)
 */
public class Gruppe extends Moebel
{
    private List moebelListe;

    /**
     * Erzeuge eine neue Moebelgruppe.
     */
    public Gruppe(int x, int y, int o)
    {
        xPosition = x;
        yPosition = y;
        farbe = "undefiniert"; // sollte nur dann definiert sein, wenn die Gruppe die Farbe definiert
        orientierung = o;
        istSichtbar = false;
        breite = 0; tiefe  = 0; // beide Werte sollten zunchst 0 sein
        moebelListe = new ArrayList();
    }
  
    /**
     * Testkonstruktor Moebelgruppe.
     */
    public Gruppe()
    {
        xPosition = 0; yPosition = 0; farbe = "gruen"; orientierung = 0; istSichtbar = false;
        breite = 0; tiefe  = 0; moebelListe = new ArrayList();
    }
  
    // Die nachfolgenden Methoden muessen ueberschrieben werden, damit sie
    // die Aufgaben weiterreichen.

    /**
     * Berechnet das zu zeichnende Shape anhand der gegebenen Daten
     */
    protected Shape gibAktuelleFigur(){
        return new Rectangle(xPosition, yPosition, breite, tiefe);
    }
    
    /**
     * setzeGroesse
     */
    public void setzeGroesse(int breite, int tiefe){
        this.breite = breite;
        this.tiefe = tiefe;
    }
  
    /**
     * Aendert die Farbe aller Kindobjekte
     */
    public void aendereFarbe(String neueFarbe)
    {
        for (Iterator it = moebelListe.iterator(); it.hasNext();){
            ((Moebel) it.next()).aendereFarbe(neueFarbe);
        }
    }
  
    /**
     * Mache diese Gruppe und alle Kindobjekte sichtbar.
     */
    public void zeige(){
        istSichtbar = true;
        for (Iterator it = moebelListe.iterator(); it.hasNext();){
            Object o = it.next();
            ((Moebel) o).setzeElternTransformation(gibTransform());
            ((Moebel) o).zeige();
        }
    }
  
    /**
     * Mache diese Gruppe und alle Kindobjekte unsichtbar.
     */
    public void verberge(){
        for (Iterator it = moebelListe.iterator(); it.hasNext();)
            ((Moebel) it.next()).verberge();
        istSichtbar = false;
    }

    /**
     * Zeichne alle Kind-Objekte mit ihren aktuellen Werten auf den Bildschirm.
     */
    public void zeichne(){
        for (Iterator it = moebelListe.iterator(); it.hasNext();){
            Object o = it.next();
            ((Moebel) o).setzeElternTransformation(gibTransform());
            ((Moebel) o).zeichne();
        }
    }
  

    // ---------- spezielle Kompositummethoden -------------
  
    /**
     * enthaelt
     * Prft, ob ein Index zulssig ist.
     */
    private boolean enthaelt(int index){
        if ((index>=0) && (index < moebelListe.size())) return true; else return false;
    }
  
    /**
     * enthaelt
     * prueft, ob ein Moebel in dieser Ebene des Kompositum enthalten ist.
     * (Eine vollstndige Suche wre aufwndig, da an der Wurzel begonnen
     * werden msste)
     */
    public boolean enthaelt(Moebel moebel){
        return moebelListe.contains(moebel);
    }
    
    /**
     * Haengt ein Moebel am Ende der moebelListe an.
     * Das Objekt wird in die Darstellung aufgenommen, wenn es in dieser Ebene
     * noch nicht vorhanden ist.
     */
    public void fuegeHinzu(Moebel moebel){
        if (!enthaelt(moebel)){
            moebelListe.add(moebel);
            moebel.verberge();
            moebel.setzeElternTransformation(gibTransform());
            if (istSichtbar) { zeige();};
        }
    }
  
    /**
     * Entfernt das Moebel an der Position i der moebelListe.
     * Wenn i unzulaessig ist, passiert nichts.
     * Das Objekt wird auch aus der Darstellung entfernt.
     */
    public void entferne(int i){
        if (enthaelt(i)){
            ((Moebel) moebelListe.get(i)).verberge();
            ((Moebel) moebelListe.get(i)).setzeElternTransformation(null);
            moebelListe.remove(i);
        }
    }
  
    /**
     * Entfernt ein Moebel aus der moebelListe.
     * Wenn es nicht existiert, passiert nichts.
     */
    public void entferne(Moebel moebel){
        if (enthaelt(moebel)){
            moebel.verberge();
            moebelListe.remove(moebel);
            moebel.setzeElternTransformation(null);
        }
    }
  
    /**
     * Gibt das Kindobjekt von der bergebenen Position zurck.
     * Ist die Position unzulssig, wird null zurckgegeben.
     * Wird bei uns nicht bentigt.
     */
    public Moebel gibKindObjekt(int anPosition){
        if (enthaelt(anPosition)){
            return (Moebel) moebelListe.get(anPosition);
        }
        else return null; 
    }
      
}
